/*
 * $Id: sound.c,v 1.2 2006/03/22 19:21:45 piru Exp $
 */
#define AROS_ALMOST_COMPATIBLE
#include "global.h"
#include "sample.h"
#include <proto/iffparse.h>
#include <datatypes/soundclass.h>
#include <clib/alib_protos.h>
#include <aros/asmcall.h>

#define DEBUG 0
#include <aros/debug.h>

#include <intuition/intuitionbase.h>

struct IntuitionBase *IntuitionBase;

static LONG SetSound(struct SoundPrefs *sprefs);

BOOL DoVisualBeep = TRUE;
BOOL DoAudioBeep /*= FALSE*/;
UWORD Type;
UWORD Volume;
UWORD Period;
UWORD Duration; /* for beep only */

struct Sample *Beep;
Object *Sound;

AROS_UFH1(LONG, SoundPrefs_Handler,
		  AROS_UFHA(struct IFFHandle *, iff, A2))
{
	AROS_LIBFUNC_INIT

	struct SoundPrefs	sprefs;

	D(bug("IPrefs: SoundPrefs\n"));


	if (ReadChunkBytes(iff, &sprefs, sizeof(sprefs)) == sizeof(sprefs))
	{
		D(bug("IPrefs: DisplayQueue 0x%lx AudioQueue 0x%lx AudioType 0x%lx AudioVolume 0x%lx AudioPeriod 0x%lx AudioDuration 0x%lx\n",
		      (ULONG) sprefs.sop_DisplayQueue,
		      (ULONG) sprefs.sop_AudioQueue,
		      (ULONG) sprefs.sop_AudioType,
		      (ULONG) sprefs.sop_AudioVolume,
		      (ULONG) sprefs.sop_AudioPeriod,
		      (ULONG) sprefs.sop_AudioDuration));

		D(bug("IPrefs: Name <%s>\n",
		      sprefs.sop_AudioFileName));

		if (!SetSound(&sprefs))
		{
			/* error req */
		}
	}
	else
	{
		D(bug("IPrefs: can't read prefs\n"));
	}

	return 0;

	AROS_LIBFUNC_EXIT
}

static struct Library *ObtainDTBase(void)
{
	if (DataTypesBase == NULL)
	{
		DataTypesBase = OpenLibrary("datatypes.library", 0);
	}

	return DataTypesBase;
}

static void ReleaseDTBase(void)
{
	if (DataTypesBase)
	{
		CloseLibrary(DataTypesBase);

		DataTypesBase = NULL;
	}
}

static UWORD o_Period;
static UWORD o_Duration;
static UBYTE o_AudioFileName[256];

static LONG SetSound(struct SoundPrefs *sprefs)
{
	LONG res = TRUE;

	DoVisualBeep = sprefs->sop_DisplayQueue ? TRUE : FALSE;
	DoAudioBeep  = sprefs->sop_AudioQueue ? TRUE : FALSE;

	if (DoAudioBeep)
	{
		Type   = sprefs->sop_AudioType;

		Volume = sprefs->sop_AudioVolume;
		if (Volume > 64)
		{
			Volume = 64;
		}

		Period = sprefs->sop_AudioPeriod;  /* It's 3135 - rate, rate is from 1 to 3000 */
		if (Period < 73) /* 135 */
		{
			Period = 73; /* 135 */
		}
		else if (Period > 3134)
		{
			Period = 3134;
		}

		Duration = sprefs->sop_AudioDuration;
		if (Duration < 1)
		{
			Duration = 1;
		}
		else if (Duration > 100)
		{
			Duration = 100;
		}
	}

	/* If volume is 0, don't bother playing anything */
	if (Volume == 0)
	{
		DoAudioBeep = FALSE;
	}

	if (Beep &&
	    (!DoAudioBeep ||
	     Type != SPTYPE_BEEP ||
	     Period != o_Period ||
	     Duration != o_Duration))
	{
		DeleteSample(Beep);
		Beep = NULL;
	}

	if (Sound &&
	    (!DoAudioBeep ||
	     Type != SPTYPE_SAMPLE ||
	     strcmp(sprefs->sop_AudioFileName, o_AudioFileName) != 0))
	{
		DisposeDTObject(Sound);
		ReleaseDTBase();
		Sound = NULL;
	}

	if (DoAudioBeep)
	{
		switch (Type)
		{
			case SPTYPE_BEEP:
			default:
				if (!Beep)
				{
					Beep = CreateBeepSample(Period, Duration);
					if (Beep)
					{
						o_Period   = Period;
						o_Duration = Duration;
					}
					else
					{
						res = FALSE;
						break;
					}
				}
				break;

			case SPTYPE_SAMPLE:
				if (!Sound)
				{
					if (ObtainDTBase())
					{
						sprefs->sop_AudioFileName[sizeof(sprefs->sop_AudioFileName) - 1] = '\0';

						Sound = NewDTObject(sprefs->sop_AudioFileName,
						                    DTA_GroupID, GID_SOUND,
						                    TAG_DONE);
						if (Sound)
						{
							strcpy(o_AudioFileName, sprefs->sop_AudioFileName);
						}
						else
						{
							ReleaseDTBase();
							res = FALSE;
							break;
						}
					}
					else
					{
						res = FALSE;
						break;
					}
				}
				break;
		}
	}

	return res;
}

void PlayBeep(void)
{
	D(bug("PlayBeep: Type %lu Volume %lu Period %lu Duration %lu\n", Type, Volume, Period, Duration));

	switch (Type)
	{
		case SPTYPE_BEEP:
			D(bug("PlayBeep: Beep 0x%lx Volume %lu Period %lu\n", Beep, Volume, Period));
			if (Beep)
			{
				PlaySample(Beep, Volume, Period);
			}
			break;

		case SPTYPE_SAMPLE:
			D(bug("PlayBeep: Sound 0x%lx Volume %lu Period %lu\n", Sound, Volume, Period));
			if (Sound)
			{
				/* Sound != NULL guarantees DataTypesBase */

				SetDTAttrs(Sound, NULL, NULL,
				           SDTA_Period, Period,
				           SDTA_Volume, Volume,
				           TAG_DONE);

				DoMethod(Sound, DTM_TRIGGER, 0, STM_PLAY, 0);
			}
			break;
	}
}


static void newDisplayBeep(void);
void (*oldDisplayBeep)(void);
#define LVODisplayBeep -96

void InstallDisplayBeepPatch(void)
{
	static const struct TagItem patchtags[] =
	{
		{SETFUNCTAG_MACHINE, MACHINE_PPC},
		{SETFUNCTAG_TYPE,    SETFUNCTYPE_NORMALNR},
		{SETFUNCTAG_IDNAME,  (ULONG) "IPrefs DisplayBeep Patch"},
		{TAG_DONE, }
	};

	Forbid();

	oldDisplayBeep = (void (*)(void)) (((char *) IntuitionBase) + LVODisplayBeep);
	oldDisplayBeep = (void (*)(void)) NewSetFunction(&IntuitionBase->LibNode,
	                                                (APTR) newDisplayBeep,
	                                                LVODisplayBeep,
	                                                (struct TagItem *) patchtags);
	Permit();
}

static void newDisplayBeep(void)
{
	struct Screen *screen = (APTR) REG_A0;
	struct IntuitionBase *IntuitionBase = (APTR) REG_A6;

	if (DoAudioBeep)
	{
		/*
		 * Notify mainloop to play the sound
		 */
		Signal(&HandlerProcess->pr_Task, SIGBREAKF_CTRL_E);
	}

	if (DoVisualBeep)
	{
		REG_A1 = (ULONG) screen;
		REG_A6 = (ULONG) IntuitionBase;
		MyEmulHandle->EmulCallDirect68k(oldDisplayBeep);
	}
}
